library(loadeR)
library(transformeR)
library(ggplot2)
library(RColorBrewer)
library(maps)
#library(climate4R.value)
library(Kendall)

rm(list=ls()) ; graphics.off() ; gc()

#Directory information
setwd("~/Documents/ETCCDI_eobs/")
OUTPUTS <- "OUTPUTS/"

#Useful information
ETCCDI_index <- c("Rx1day","Rx5day","TN10p","TN90p","TNn","TNx","TX10p","TX90p","TXn","TXx","CDD","CWD","CSDI","WSDI")
seasons <- list(Summer=c(6,7,8),Autumn=c(9,10,11),Winter=c(12,1,2),Spring=c(3,4,5))

calendario <- seq(as.Date("1950-01-01"),as.Date("2021-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]

#Load CPs information
load("../WardCluster_Dendrogram_D2_50km.Rdata")
k=5
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]
cp <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}
value <- NA ; value.cat <- NA ; sign <- NA ; sign.cat <- NA

for(ind in 1:length(ETCCDI_index)){
  #Load data
  files <- list.files(pattern = ETCCDI_index[ind])
  data <- list() ; data.cat <- list()
  
  for(i in 1:length(files)){
    di <- dataInventory(dataset = files[i])
    nc <- loadGridData(dataset=files[i],var=names(di))
    
    #Aggregation of annual/seasonal indices (climatology: time-series)
    if(length(nc$Dates$start)<=72){
      index2 <- aggregateGrid(nc,aggr.spatial = list(FUN="mean",na.rm=T))
      index.cat <- aggregateGrid(subsetGrid(nc,lonLim = c(0,3),latLim = c(40,43)),
                                 aggr.spatial = list(FUN="mean",na.rm=T))
  
      #Generating time-series for each CP
      for(l in 1:length(unique(vector_cluster))){
        v <- unique(vector_cluster)[l]
        cp2 <- cp
        cp2$Data[which(cp2$Data!=v)] <- 0
        cp2$Data[which(cp2$Data==v)] <- 1
        index.cp <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(1:12)))
       
        #Removing trend in CP frequency (if any)
        trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
        index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
        
        #Removing trend in ETCCDI index (if any)
        trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
        index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
        
        trendfreq.cat  <- lm(index.cat$Data~c(1:length(unique(getYearsAsINDEX(index.cat)))))[["coefficients"]][2]
        index.cat$Data <- index.cat$Data - trendfreq.cat*c(1:length(unique(getYearsAsINDEX(index.cat))))
        
        value[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
        value.cat[l] <- as.numeric(Kendall(x=index.cp$Data,y=index.cat$Data)$tau)
        
        sign[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
        sign.cat[l] <- as.numeric(Kendall(x=index.cp$Data,y=index.cat$Data)$sl)
      }
      
      data[[1]] <- data.frame(Index=value,
                              Season=rep("Annual",length(value)),
                              CP=unique(vector_cluster),
                              Sign=sign)
      data.cat[[1]] <- data.frame(Index=value.cat,
                                    Season=rep("Annual",length(value)),
                                    CP=unique(vector_cluster),
                                    Sign=sign.cat)
     }
    if(length(nc$Dates$start)>72){
      for(j in 1:length(seasons)){
        index2 <- redim(aggregateGrid(nc,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[j]])),drop=T)
        index.cat <- subsetGrid(index2,lonLim = c(0,3),latLim = c(40,43))
        
        index2 <- aggregateGrid(index2,aggr.spatial = list(FUN="mean",na.rm=T))
        index.cat <- aggregateGrid(index.cat,aggr.spatial = list(FUN="mean",na.rm=T))
        
        #Generating time-series for each CP
        for(l in 1:length(unique(vector_cluster))){
          v <- unique(vector_cluster)[l]
          cp2 <- cp
          cp2$Data[which(cp2$Data!=v)] <- 0
          cp2$Data[which(cp2$Data==v)] <- 1
        index.cp <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=seasons[[j]]))
        #Removing trend in CP frequency (if any)
        trendfreq  <- lm(index.cp$Data~c(1:length(unique(getYearsAsINDEX(index.cp)))))[["coefficients"]][2]
        index.cp$Data <- index.cp$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index.cp))))
        
        #Removing trend in ETCCDI index (if any)
        trendfreq  <- lm(index2$Data~c(1:length(unique(getYearsAsINDEX(index2)))))[["coefficients"]][2]
        index2$Data <- index2$Data - trendfreq*c(1:length(unique(getYearsAsINDEX(index2))))
        
        trendfreq.cat  <- lm(index.cat$Data~c(1:length(unique(getYearsAsINDEX(index.cat)))))[["coefficients"]][2]
        index.cat$Data <- index.cat$Data - trendfreq.cat*c(1:length(unique(getYearsAsINDEX(index.cat))))
        
        value[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$tau)
        value.cat[l] <- as.numeric(Kendall(x=index.cp$Data,y=index.cat$Data)$tau)
        
        sign[l] <- as.numeric(Kendall(x=index.cp$Data,y=index2$Data)$sl)
        sign.cat[l] <- as.numeric(Kendall(x=index.cp$Data,y=index.cat$Data)$sl)                }
        
        data[[j+1]] <- data.frame(Index=value,
                                Season=rep(names(seasons)[j],length(value)),
                                CP=unique(vector_cluster),
                                Sign=sign)
        data.cat[[j+1]] <- data.frame(Index=value.cat,
                                    Season=rep(names(seasons)[j],length(value)),
                                    CP=unique(vector_cluster),
                                    Sign=sign)
      }}
  }
  
  #Organize for ggplot plotting and saving
  datos <- do.call(rbind,data)
  datos$Season <- factor(datos$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  datos.cat <- do.call(rbind,data.cat)
  datos.cat$Season <- factor(datos.cat$Season,levels=c("Winter","Autumn","Summer","Spring","Annual"))
  
  if(length(unique(datos$Season))>1){
    datos <- datos[-which(datos$Season=="Annual"),]
    datos.cat <- datos.cat[-which(datos.cat$Season=="Annual"),]
  }
  
  mix <- rbind(datos,datos.cat)
  mix$Region <- rep(c("Iberian Peninsula","Catalunya"),each=dim(datos)[1])
  
  png(filename = paste(OUTPUTS,ETCCDI_index[ind],"_E-OBS_Correlations-Ktau_D2k1.png",sep=""),res=300,width=1500,height=1500)
  print(ggplot(data=mix,aes(x=Season,y=CP,fill=Index)) + geom_tile() + facet_wrap(~Region,nrow = 3) +
          theme_bw() + ggtitle(paste("E-OBS",names(di),sep=" ")) + 
          scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdBu")),limits=c(-1,1),
                                aesthetics = "fill",na.value = "white") + 
          geom_text(data=subset(mix,Sign<=0.05),aes(label=round(Index,digits=2)),fontface="bold")+
      geom_text(data=subset(mix,Sign>0.05),aes(label=round(Index,digits=2)))) 
  dev.off()
} 


#####################################
#Time series CPs
#Load CPs information
load("../WardCluster_Dendrogram_D2_50km.Rdata")
k=5
vector_cluster <- cutree(ward_clustering,k)[which(format(calendario,"%Y")%in%c(1950:2021))]
cp <- loadGridData(dataset = "../eobs_tasmax_spain.nc",lonLim = c(-1,1),latLim = c(35,36),var="tasmax")
cp <- aggregateGrid(cp,aggr.spatial = list(FUN="mean"))
for(l in 1:length(cp$Data)){
  cp$Data[l] <- vector_cluster[l]
}

data <- list() ; mix <- list()
for(l in 1:length(unique(vector_cluster))){
  v <- unique(vector_cluster)[l]
  cp2 <- cp
  cp2$Data[which(cp2$Data!=v)] <- 0
  cp2$Data[which(cp2$Data==v)] <- 1
  
  for(j in 1:length(seasons)){
  index.cp <- aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=seasons[[j]]))
  data[[j]] <- data.frame(Index=index.cp$Data/90*100,Years=getYearsAsINDEX(index.cp),
                     Season=rep(names(seasons)[j],length(index.cp$Data)),
                     CP=rep(v,length(index.cp$Data)))
  }
  data[[5]] <- data.frame(Index=aggregateGrid(cp2,aggr.s = list(FUN=list("sum",na.rm=T),season=c(1:12)))$Data/365*100,Years=getYearsAsINDEX(index.cp),
                          Season=rep("Annual",length(index.cp$Data)),
                          CP=rep(v,length(index.cp$Data)))
  mix[[v]] <- do.call(rbind,data)
}

palette <- brewer.pal(n=7,name = "Set1")


mix <- do.call(rbind,mix)
mix2 <- subset(mix,mix$Season!="Annual")
mix2$CP <- factor(mix2$CP,levels=c(1:k))

png(filename = paste(OUTPUTS,"CPs_TimeSeries_D2k1_cols.png",sep=""),res=600,width=4000,height=2000)
print(ggplot(data=mix2,aes(x=Years,y=Index,fill=CP)) + geom_col() + theme_bw()+facet_wrap(~Season,nrow=2)+  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  scale_x_continuous(n.breaks = 8) + scale_y_continuous(n.breaks = 6) + ylab("Relative frequency (%)")+ scale_fill_manual(values=palette[1:k]),aesthetics=c("color","fill"))
  # ggtitle(paste("CPs relative frequency","D2 k1",sep=" ")) + geom_smooth(method=lm,se=F,lwd=0.75))
dev.off()
