library(ggplot2)
library(RColorBrewer)

#Load DISO data & ErrorCycle data
load("Desktop/DISO_PR.Rdata")
DISO_PR <- datitos$DISO[which(datitos$CP=="Mean")]
GCM <- datitos$GCM[which(datitos$CP=="Mean")]

load("Desktop/DISO_TASMIN.Rdata")
DISO_TASMIN <- datitos$DISO[which(datitos$CP=="Mean")]

load("Desktop/DISO_TASMAX.Rdata")
DISO_TASMAX <- datitos$DISO[which(datitos$CP=="Mean")]

DISO_mean <- ((0.5*DISO_PR)+(0.25*DISO_TASMAX)+(0.25*DISO_TASMIN))

load("Desktop/ErrorCycle.Rdata")
data <- data[which(data$GCM %in% GCM),]
ErrorCycle <- data$Error[which(data$CP=="Mean")]

datos <- data.frame(x=DISO_mean,y=ErrorCycle,GCM=paste(c(1:29),GCM),Number=c(1:29))
datos$GCM <- factor(datos$GCM,levels=paste(c(1:29),GCM))

datos$y[which(datos$GCM=="NESM3")] <- 14.99

p <- ggplot(data=datos,aes(x=x,y=y,colour=GCM)) +
  geom_point(aes(size=1.2)) + theme_bw() + coord_cartesian(expand = F)  + geom_text(aes(x=x,y=y,label=Number),colour="black")+
  xlim(c(0,0.75)) + ylim(c(0,15)) + xlab("DISO mean") + ylab("ErrorCycle")+
  scale_colour_manual(values=rainbow(n=length(GCM))) + theme(legend.position = "bottom")

ggsave(p,filename="Desktop/dispersion_DISO_ErrorCycle.png",device="png",
       dpi=600,width=7,height = 7)                                  

####
x <- seq(0,0.75,by=0.01)
x <- x/mean(x)
y <- seq(0,15,by=0.01)
y <- mean(y)

#Euclidean distance
dist <- matrix(NA,length(x),length(y))
for(i in 1:length(x)){
  for(j in 1:length(y)){
    dist[i,j] <- round(sqrt((x[i]^2)+(y[j]^2)),digits=3)
  }
}

data <- data.frame(x=rep(x,length(y)),y=rep(y,each=length(x)),dist=as.numeric(dist))
p30 <- quantile(data$dist,probs = 0.3) #34
p60 <- quantile(data$dist,probs=0.6) #46

p <- ggplot(data=datos,aes(x=x,y=y)) +
  geom_point(aes(size=1.2,colour=GCM)) + theme_bw() + coord_cartesian(expand = F)  + geom_text(aes(x=x,y=y,label=Number),colour="black")+
  xlim(c(0,0.75)) + ylim(c(0,15)) + xlab("DISO mean") + ylab("ErrorCycle")+
  scale_colour_manual(values=rainbow(n=length(GCM))) + theme(legend.position = "bottom")+
  geom_tile(data=data,aes(x=x,y=y,fill=dist))


#which(dist==p60)

